//*******************************************************************************************************************
//
//  Author(s)...: DOOMSDAY
//
//  Target(s)...: STM8l
//
//  Compiler....: IAR EW for STM8
//
//  Description.:      STM8l-Discovery
//
//  Data........: 15.11.11 
//
//*******************************************************************************************************************

#include "LCD.h"
#include "iostm8l152c6.h"



//*******************************************************************************************************************
/*
     A
     _  ----------
COL |_| |\   |J  /|
       F| H  |  K |B
     _  |  \ | /  |
COL |_| --G-- --M--
        |   /| \  |
       E|  Q |  N |C
     _  | /  |P  \|
DP  |_| -----------
     D
       
       */
//  
const unsigned int number_mask[] = 
{
  //0
  (1<<A_)|(1<<B_)|(1<<C_)|(1<<D_)|(1<<E_)|(1<<F_)|(0<<G_)|(0<<H_)|(0<<J_)|(1<<K_)|(0<<M_)|(0<<N_)|(0<<P_)|(1<<Q_)|(0<<DP)|(0<<COL), 
  //1
  (0<<A_)|(1<<B_)|(1<<C_)|(0<<D_)|(0<<E_)|(0<<F_)|(0<<G_)|(0<<H_)|(0<<J_)|(0<<K_)|(0<<M_)|(0<<N_)|(0<<P_)|(0<<Q_)|(0<<DP)|(0<<COL), 
  //2
  (1<<A_)|(1<<B_)|(0<<C_)|(1<<D_)|(1<<E_)|(0<<F_)|(1<<G_)|(0<<H_)|(0<<J_)|(0<<K_)|(1<<M_)|(0<<N_)|(0<<P_)|(0<<Q_)|(0<<DP)|(0<<COL),  
  //3
  (1<<A_)|(1<<B_)|(1<<C_)|(1<<D_)|(0<<E_)|(0<<F_)|(1<<G_)|(0<<H_)|(0<<J_)|(0<<K_)|(1<<M_)|(0<<N_)|(0<<P_)|(0<<Q_)|(0<<DP)|(0<<COL), 
  //4
  (0<<A_)|(1<<B_)|(1<<C_)|(0<<D_)|(0<<E_)|(1<<F_)|(1<<G_)|(0<<H_)|(0<<J_)|(0<<K_)|(1<<M_)|(0<<N_)|(0<<P_)|(0<<Q_)|(0<<DP)|(0<<COL), 
  //5
  (1<<A_)|(0<<B_)|(1<<C_)|(1<<D_)|(0<<E_)|(1<<F_)|(1<<G_)|(0<<H_)|(0<<J_)|(0<<K_)|(1<<M_)|(0<<N_)|(0<<P_)|(0<<Q_)|(0<<DP)|(0<<COL), 
  //6
  (1<<A_)|(0<<B_)|(1<<C_)|(1<<D_)|(1<<E_)|(1<<F_)|(1<<G_)|(0<<H_)|(0<<J_)|(0<<K_)|(1<<M_)|(0<<N_)|(0<<P_)|(0<<Q_)|(0<<DP)|(0<<COL), 
  //7
  (1<<A_)|(1<<B_)|(1<<C_)|(0<<D_)|(0<<E_)|(1<<F_)|(0<<G_)|(0<<H_)|(0<<J_)|(0<<K_)|(0<<M_)|(0<<N_)|(0<<P_)|(0<<Q_)|(0<<DP)|(0<<COL), 
  //8
  (1<<A_)|(1<<B_)|(1<<C_)|(1<<D_)|(1<<E_)|(1<<F_)|(1<<G_)|(0<<H_)|(0<<J_)|(0<<K_)|(1<<M_)|(0<<N_)|(0<<P_)|(0<<Q_)|(0<<DP)|(0<<COL), 
  //9
  (1<<A_)|(1<<B_)|(1<<C_)|(1<<D_)|(0<<E_)|(1<<F_)|(1<<G_)|(0<<H_)|(0<<J_)|(0<<K_)|(1<<M_)|(0<<N_)|(0<<P_)|(0<<Q_)|(0<<DP)|(0<<COL)
};

//   
const unsigned int char_mask[] = 
{
  //A
  (1<<A_)|(1<<B_)|(1<<C_)|(0<<D_)|(1<<E_)|(1<<F_)|(1<<G_)|(0<<H_)|(0<<J_)|(0<<K_)|(1<<M_)|(0<<N_)|(0<<P_)|(0<<Q_)|(0<<DP)|(0<<COL), 
  //B
  (1<<A_)|(0<<B_)|(1<<C_)|(1<<D_)|(1<<E_)|(1<<F_)|(1<<G_)|(0<<H_)|(0<<J_)|(1<<K_)|(1<<M_)|(0<<N_)|(0<<P_)|(0<<Q_)|(0<<DP)|(0<<COL), 
  //C
  (1<<A_)|(0<<B_)|(0<<C_)|(1<<D_)|(1<<E_)|(1<<F_)|(0<<G_)|(0<<H_)|(0<<J_)|(0<<K_)|(0<<M_)|(0<<N_)|(0<<P_)|(0<<Q_)|(0<<DP)|(0<<COL),  
  //D
  (1<<A_)|(1<<B_)|(1<<C_)|(1<<D_)|(0<<E_)|(0<<F_)|(0<<G_)|(0<<H_)|(1<<J_)|(0<<K_)|(0<<M_)|(0<<N_)|(1<<P_)|(0<<Q_)|(0<<DP)|(0<<COL), 
  //E
  (1<<A_)|(0<<B_)|(0<<C_)|(1<<D_)|(1<<E_)|(1<<F_)|(1<<G_)|(0<<H_)|(0<<J_)|(0<<K_)|(1<<M_)|(0<<N_)|(0<<P_)|(0<<Q_)|(0<<DP)|(0<<COL), 
  //F
  (1<<A_)|(0<<B_)|(0<<C_)|(0<<D_)|(1<<E_)|(1<<F_)|(1<<G_)|(0<<H_)|(0<<J_)|(0<<K_)|(1<<M_)|(0<<N_)|(0<<P_)|(0<<Q_)|(0<<DP)|(0<<COL), 
  //G
  (1<<A_)|(0<<B_)|(1<<C_)|(1<<D_)|(1<<E_)|(1<<F_)|(0<<G_)|(0<<H_)|(0<<J_)|(0<<K_)|(1<<M_)|(0<<N_)|(0<<P_)|(0<<Q_)|(0<<DP)|(0<<COL), 
  //H
  (0<<A_)|(1<<B_)|(1<<C_)|(0<<D_)|(1<<E_)|(1<<F_)|(1<<G_)|(0<<H_)|(0<<J_)|(0<<K_)|(1<<M_)|(0<<N_)|(0<<P_)|(0<<Q_)|(0<<DP)|(0<<COL), 
  //I
  (0<<A_)|(0<<B_)|(0<<C_)|(0<<D_)|(0<<E_)|(0<<F_)|(0<<G_)|(0<<H_)|(1<<J_)|(0<<K_)|(0<<M_)|(0<<N_)|(1<<P_)|(0<<Q_)|(0<<DP)|(0<<COL), 
  //J
  (0<<A_)|(1<<B_)|(1<<C_)|(1<<D_)|(1<<E_)|(0<<F_)|(0<<G_)|(0<<H_)|(0<<J_)|(0<<K_)|(0<<M_)|(0<<N_)|(0<<P_)|(0<<Q_)|(0<<DP)|(0<<COL), 
  //K
  (0<<A_)|(0<<B_)|(0<<C_)|(0<<D_)|(1<<E_)|(1<<F_)|(1<<G_)|(0<<H_)|(0<<J_)|(1<<K_)|(0<<M_)|(1<<N_)|(0<<P_)|(0<<Q_)|(0<<DP)|(0<<COL), 
  //L
  (0<<A_)|(0<<B_)|(0<<C_)|(1<<D_)|(1<<E_)|(1<<F_)|(0<<G_)|(0<<H_)|(0<<J_)|(0<<K_)|(0<<M_)|(0<<N_)|(0<<P_)|(0<<Q_)|(0<<DP)|(0<<COL), 
  //M
  (0<<A_)|(1<<B_)|(1<<C_)|(0<<D_)|(1<<E_)|(1<<F_)|(0<<G_)|(1<<H_)|(0<<J_)|(1<<K_)|(0<<M_)|(0<<N_)|(0<<P_)|(0<<Q_)|(0<<DP)|(0<<COL), 
  //N
  (0<<A_)|(1<<B_)|(1<<C_)|(0<<D_)|(1<<E_)|(1<<F_)|(0<<G_)|(1<<H_)|(0<<J_)|(0<<K_)|(0<<M_)|(1<<N_)|(0<<P_)|(0<<Q_)|(0<<DP)|(0<<COL),  
  //O
  (1<<A_)|(1<<B_)|(1<<C_)|(1<<D_)|(1<<E_)|(1<<F_)|(0<<G_)|(0<<H_)|(0<<J_)|(0<<K_)|(0<<M_)|(0<<N_)|(0<<P_)|(0<<Q_)|(0<<DP)|(0<<COL), 
  //P
  (1<<A_)|(1<<B_)|(0<<C_)|(0<<D_)|(1<<E_)|(1<<F_)|(1<<G_)|(0<<H_)|(0<<J_)|(0<<K_)|(1<<M_)|(0<<N_)|(0<<P_)|(0<<Q_)|(0<<DP)|(0<<COL), 
  //Q
  (1<<A_)|(1<<B_)|(1<<C_)|(1<<D_)|(1<<E_)|(1<<F_)|(0<<G_)|(0<<H_)|(0<<J_)|(0<<K_)|(0<<M_)|(1<<N_)|(0<<P_)|(0<<Q_)|(0<<DP)|(0<<COL), 
  //R
  (1<<A_)|(1<<B_)|(0<<C_)|(0<<D_)|(1<<E_)|(1<<F_)|(1<<G_)|(0<<H_)|(0<<J_)|(0<<K_)|(1<<M_)|(1<<N_)|(0<<P_)|(0<<Q_)|(0<<DP)|(0<<COL), 
  //S
  (1<<A_)|(0<<B_)|(1<<C_)|(1<<D_)|(0<<E_)|(1<<F_)|(1<<G_)|(0<<H_)|(0<<J_)|(0<<K_)|(1<<M_)|(0<<N_)|(0<<P_)|(0<<Q_)|(0<<DP)|(0<<COL), 
  //T
  (1<<A_)|(0<<B_)|(0<<C_)|(0<<D_)|(0<<E_)|(0<<F_)|(0<<G_)|(0<<H_)|(1<<J_)|(0<<K_)|(0<<M_)|(0<<N_)|(1<<P_)|(0<<Q_)|(0<<DP)|(0<<COL), 
  //U
  (0<<A_)|(1<<B_)|(1<<C_)|(1<<D_)|(1<<E_)|(1<<F_)|(0<<G_)|(0<<H_)|(0<<J_)|(0<<K_)|(0<<M_)|(0<<N_)|(0<<P_)|(0<<Q_)|(0<<DP)|(0<<COL),   
  //V
  (0<<A_)|(0<<B_)|(0<<C_)|(0<<D_)|(1<<E_)|(1<<F_)|(0<<G_)|(0<<H_)|(0<<J_)|(1<<K_)|(0<<M_)|(0<<N_)|(0<<P_)|(1<<Q_)|(0<<DP)|(0<<COL), 
  //W
  (0<<A_)|(1<<B_)|(1<<C_)|(0<<D_)|(1<<E_)|(1<<F_)|(0<<G_)|(0<<H_)|(0<<J_)|(0<<K_)|(0<<M_)|(1<<N_)|(0<<P_)|(1<<Q_)|(0<<DP)|(0<<COL), 
  //X
  (0<<A_)|(0<<B_)|(0<<C_)|(0<<D_)|(0<<E_)|(0<<F_)|(0<<G_)|(1<<H_)|(0<<J_)|(1<<K_)|(0<<M_)|(1<<N_)|(0<<P_)|(1<<Q_)|(0<<DP)|(0<<COL), 
  //Y
  (0<<A_)|(0<<B_)|(0<<C_)|(0<<D_)|(0<<E_)|(0<<F_)|(0<<G_)|(1<<H_)|(0<<J_)|(1<<K_)|(0<<M_)|(0<<N_)|(1<<P_)|(0<<Q_)|(0<<DP)|(0<<COL), 
  //Z  
  (1<<A_)|(0<<B_)|(0<<C_)|(1<<D_)|(0<<E_)|(0<<F_)|(0<<G_)|(0<<H_)|(0<<J_)|(1<<K_)|(0<<M_)|(0<<N_)|(0<<P_)|(1<<Q_)|(0<<DP)|(0<<COL)
};

//  
const unsigned int rus_char_mask[] = 
{
  //A
  (1<<A_)|(1<<B_)|(1<<C_)|(0<<D_)|(1<<E_)|(1<<F_)|(1<<G_)|(0<<H_)|(0<<J_)|(0<<K_)|(1<<M_)|(0<<N_)|(0<<P_)|(0<<Q_)|(0<<DP)|(0<<COL), 
  //
  (1<<A_)|(0<<B_)|(1<<C_)|(1<<D_)|(1<<E_)|(1<<F_)|(1<<G_)|(0<<H_)|(0<<J_)|(0<<K_)|(1<<M_)|(0<<N_)|(0<<P_)|(0<<Q_)|(0<<DP)|(0<<COL), 
  //
  (1<<A_)|(0<<B_)|(1<<C_)|(1<<D_)|(1<<E_)|(1<<F_)|(1<<G_)|(0<<H_)|(0<<J_)|(1<<K_)|(1<<M_)|(0<<N_)|(0<<P_)|(0<<Q_)|(0<<DP)|(0<<COL),  
  //
  (1<<A_)|(0<<B_)|(0<<C_)|(0<<D_)|(1<<E_)|(1<<F_)|(0<<G_)|(0<<H_)|(0<<J_)|(0<<K_)|(0<<M_)|(0<<N_)|(0<<P_)|(0<<Q_)|(0<<DP)|(0<<COL), 
  //
  (0<<A_)|(1<<B_)|(1<<C_)|(1<<D_)|(0<<E_)|(0<<F_)|(0<<G_)|(0<<H_)|(0<<J_)|(1<<K_)|(0<<M_)|(0<<N_)|(0<<P_)|(1<<Q_)|(0<<DP)|(0<<COL), 
  //
  (1<<A_)|(0<<B_)|(0<<C_)|(1<<D_)|(1<<E_)|(1<<F_)|(1<<G_)|(0<<H_)|(0<<J_)|(0<<K_)|(1<<M_)|(0<<N_)|(0<<P_)|(0<<Q_)|(0<<DP)|(0<<COL), 
  //
  (0<<A_)|(1<<B_)|(1<<C_)|(0<<D_)|(1<<E_)|(1<<F_)|(1<<G_)|(0<<H_)|(1<<J_)|(0<<K_)|(1<<M_)|(0<<N_)|(1<<P_)|(0<<Q_)|(0<<DP)|(0<<COL), 
  //
  (1<<A_)|(0<<B_)|(0<<C_)|(1<<D_)|(0<<E_)|(0<<F_)|(1<<G_)|(0<<H_)|(0<<J_)|(1<<K_)|(0<<M_)|(1<<N_)|(0<<P_)|(0<<Q_)|(0<<DP)|(0<<COL), 
  //
  (0<<A_)|(1<<B_)|(1<<C_)|(0<<D_)|(1<<E_)|(1<<F_)|(0<<G_)|(0<<H_)|(0<<J_)|(1<<K_)|(0<<M_)|(0<<N_)|(0<<P_)|(1<<Q_)|(0<<DP)|(0<<COL), 
  //
  (1<<A_)|(1<<B_)|(1<<C_)|(0<<D_)|(1<<E_)|(1<<F_)|(0<<G_)|(0<<H_)|(0<<J_)|(1<<K_)|(0<<M_)|(0<<N_)|(0<<P_)|(1<<Q_)|(0<<DP)|(0<<COL), 
  //
  (0<<A_)|(0<<B_)|(0<<C_)|(0<<D_)|(1<<E_)|(1<<F_)|(1<<G_)|(0<<H_)|(0<<J_)|(1<<K_)|(0<<M_)|(1<<N_)|(0<<P_)|(0<<Q_)|(0<<DP)|(0<<COL), 
  //
  (0<<A_)|(1<<B_)|(1<<C_)|(0<<D_)|(0<<E_)|(0<<F_)|(0<<G_)|(0<<H_)|(0<<J_)|(1<<K_)|(0<<M_)|(0<<N_)|(0<<P_)|(1<<Q_)|(0<<DP)|(0<<COL), 
  //
  (0<<A_)|(1<<B_)|(1<<C_)|(0<<D_)|(1<<E_)|(1<<F_)|(0<<G_)|(1<<H_)|(0<<J_)|(1<<K_)|(0<<M_)|(0<<N_)|(0<<P_)|(0<<Q_)|(0<<DP)|(0<<COL), 
  //
  (0<<A_)|(1<<B_)|(1<<C_)|(0<<D_)|(1<<E_)|(1<<F_)|(1<<G_)|(0<<H_)|(0<<J_)|(0<<K_)|(1<<M_)|(0<<N_)|(0<<P_)|(0<<Q_)|(0<<DP)|(0<<COL),  
  //
  (1<<A_)|(1<<B_)|(1<<C_)|(1<<D_)|(1<<E_)|(1<<F_)|(0<<G_)|(0<<H_)|(0<<J_)|(0<<K_)|(0<<M_)|(0<<N_)|(0<<P_)|(0<<Q_)|(0<<DP)|(0<<COL), 
  //
  (1<<A_)|(1<<B_)|(1<<C_)|(0<<D_)|(1<<E_)|(1<<F_)|(0<<G_)|(0<<H_)|(0<<J_)|(0<<K_)|(0<<M_)|(0<<N_)|(0<<P_)|(0<<Q_)|(0<<DP)|(0<<COL), 
  //
  (1<<A_)|(1<<B_)|(0<<C_)|(0<<D_)|(1<<E_)|(1<<F_)|(1<<G_)|(0<<H_)|(0<<J_)|(0<<K_)|(1<<M_)|(0<<N_)|(0<<P_)|(0<<Q_)|(0<<DP)|(0<<COL), 
  //
  (1<<A_)|(0<<B_)|(0<<C_)|(1<<D_)|(1<<E_)|(1<<F_)|(0<<G_)|(0<<H_)|(0<<J_)|(0<<K_)|(0<<M_)|(0<<N_)|(0<<P_)|(0<<Q_)|(0<<DP)|(0<<COL), 
  //
  (1<<A_)|(0<<B_)|(0<<C_)|(0<<D_)|(0<<E_)|(0<<F_)|(0<<G_)|(0<<H_)|(1<<J_)|(0<<K_)|(0<<M_)|(0<<N_)|(1<<P_)|(0<<Q_)|(0<<DP)|(0<<COL), 
  //
  (0<<A_)|(1<<B_)|(1<<C_)|(1<<D_)|(0<<E_)|(1<<F_)|(1<<G_)|(0<<H_)|(0<<J_)|(0<<K_)|(1<<M_)|(0<<N_)|(0<<P_)|(0<<Q_)|(0<<DP)|(0<<COL), 
  //
  (1<<A_)|(1<<B_)|(0<<C_)|(0<<D_)|(0<<E_)|(1<<F_)|(1<<G_)|(0<<H_)|(1<<J_)|(0<<K_)|(1<<M_)|(0<<N_)|(1<<P_)|(0<<Q_)|(0<<DP)|(0<<COL),   
  //
  (0<<A_)|(0<<B_)|(0<<C_)|(0<<D_)|(0<<E_)|(0<<F_)|(0<<G_)|(1<<H_)|(0<<J_)|(1<<K_)|(0<<M_)|(1<<N_)|(0<<P_)|(1<<Q_)|(0<<DP)|(0<<COL), 
  //
  (0<<A_)|(0<<B_)|(0<<C_)|(1<<D_)|(1<<E_)|(1<<F_)|(0<<G_)|(0<<H_)|(1<<J_)|(0<<K_)|(0<<M_)|(0<<N_)|(1<<P_)|(0<<Q_)|(0<<DP)|(0<<COL), 
  //
  (0<<A_)|(1<<B_)|(1<<C_)|(0<<D_)|(0<<E_)|(1<<F_)|(1<<G_)|(0<<H_)|(0<<J_)|(0<<K_)|(1<<M_)|(0<<N_)|(0<<P_)|(0<<Q_)|(0<<DP)|(0<<COL), 
  //
  (0<<A_)|(1<<B_)|(1<<C_)|(1<<D_)|(1<<E_)|(1<<F_)|(0<<G_)|(0<<H_)|(1<<J_)|(0<<K_)|(0<<M_)|(0<<N_)|(1<<P_)|(0<<Q_)|(0<<DP)|(0<<COL), 
  //
  (0<<A_)|(1<<B_)|(1<<C_)|(1<<D_)|(1<<E_)|(1<<F_)|(0<<G_)|(0<<H_)|(1<<J_)|(0<<K_)|(0<<M_)|(0<<N_)|(1<<P_)|(0<<Q_)|(0<<DP)|(0<<COL),
  //
  (0<<A_)|(0<<B_)|(1<<C_)|(1<<D_)|(0<<E_)|(1<<F_)|(0<<G_)|(1<<H_)|(0<<J_)|(0<<K_)|(1<<M_)|(0<<N_)|(0<<P_)|(1<<Q_)|(0<<DP)|(0<<COL), 
  //
  (0<<A_)|(1<<B_)|(1<<C_)|(0<<D_)|(1<<E_)|(1<<F_)|(1<<G_)|(0<<H_)|(0<<J_)|(0<<K_)|(0<<M_)|(0<<N_)|(1<<P_)|(1<<Q_)|(0<<DP)|(0<<COL), 
  //
  (0<<A_)|(0<<B_)|(1<<C_)|(1<<D_)|(0<<E_)|(0<<F_)|(0<<G_)|(1<<H_)|(0<<J_)|(0<<K_)|(1<<M_)|(0<<N_)|(0<<P_)|(1<<Q_)|(0<<DP)|(0<<COL), 
  //
  (1<<A_)|(1<<B_)|(1<<C_)|(1<<D_)|(0<<E_)|(0<<F_)|(0<<G_)|(0<<H_)|(0<<J_)|(0<<K_)|(1<<M_)|(0<<N_)|(0<<P_)|(0<<Q_)|(0<<DP)|(0<<COL), 
  //  
  (0<<A_)|(1<<B_)|(1<<C_)|(0<<D_)|(1<<E_)|(1<<F_)|(1<<G_)|(0<<H_)|(0<<J_)|(1<<K_)|(0<<M_)|(1<<N_)|(0<<P_)|(0<<Q_)|(0<<DP)|(0<<COL),
  //
  (1<<A_)|(1<<B_)|(1<<C_)|(0<<D_)|(0<<E_)|(1<<F_)|(1<<G_)|(0<<H_)|(0<<J_)|(0<<K_)|(1<<M_)|(0<<N_)|(0<<P_)|(1<<Q_)|(0<<DP)|(0<<COL)
};
// 
const unsigned int spec_char_mask[] =
{
  //$
  (1<<A_)|(0<<B_)|(1<<C_)|(1<<D_)|(0<<E_)|(1<<F_)|(1<<G_)|(0<<H_)|(1<<J_)|(0<<K_)|(1<<M_)|(0<<N_)|(1<<P_)|(0<<Q_)|(0<<DP)|(0<<COL),
  //(
  (0<<A_)|(0<<B_)|(0<<C_)|(0<<D_)|(0<<E_)|(0<<F_)|(0<<G_)|(0<<H_)|(0<<J_)|(1<<K_)|(0<<M_)|(1<<N_)|(0<<P_)|(0<<Q_)|(0<<DP)|(0<<COL),
  //)
  (0<<A_)|(0<<B_)|(0<<C_)|(0<<D_)|(0<<E_)|(0<<F_)|(0<<G_)|(1<<H_)|(0<<J_)|(0<<K_)|(0<<M_)|(0<<N_)|(0<<P_)|(1<<Q_)|(0<<DP)|(0<<COL),
  //*
  (0<<A_)|(0<<B_)|(0<<C_)|(0<<D_)|(0<<E_)|(0<<F_)|(1<<G_)|(1<<H_)|(1<<J_)|(1<<K_)|(1<<M_)|(1<<N_)|(1<<P_)|(1<<Q_)|(0<<DP)|(0<<COL),
  //+
  (0<<A_)|(0<<B_)|(0<<C_)|(0<<D_)|(0<<E_)|(0<<F_)|(1<<G_)|(0<<H_)|(1<<J_)|(0<<K_)|(1<<M_)|(0<<N_)|(1<<P_)|(0<<Q_)|(0<<DP)|(0<<COL),
  //-
  (0<<A_)|(0<<B_)|(0<<C_)|(0<<D_)|(0<<E_)|(0<<F_)|(1<<G_)|(0<<H_)|(0<<J_)|(0<<K_)|(1<<M_)|(0<<N_)|(0<<P_)|(0<<Q_)|(0<<DP)|(0<<COL),
  //=
  (0<<A_)|(0<<B_)|(0<<C_)|(1<<D_)|(0<<E_)|(0<<F_)|(1<<G_)|(0<<H_)|(0<<J_)|(0<<K_)|(1<<M_)|(0<<N_)|(0<<P_)|(0<<Q_)|(0<<DP)|(0<<COL),
  //m
  (0<<A_)|(0<<B_)|(1<<C_)|(0<<D_)|(1<<E_)|(0<<F_)|(1<<G_)|(0<<H_)|(0<<J_)|(0<<K_)|(1<<M_)|(0<<N_)|(1<<P_)|(0<<Q_)|(0<<DP)|(0<<COL),
  //u
  (0<<A_)|(0<<B_)|(0<<C_)|(0<<D_)|(0<<E_)|(1<<F_)|(1<<G_)|(0<<H_)|(0<<J_)|(1<<K_)|(0<<M_)|(1<<N_)|(0<<P_)|(0<<Q_)|(0<<DP)|(0<<COL),
  //n
  (0<<A_)|(0<<B_)|(1<<C_)|(0<<D_)|(1<<E_)|(0<<F_)|(1<<G_)|(0<<H_)|(0<<J_)|(0<<K_)|(1<<M_)|(0<<N_)|(0<<P_)|(0<<Q_)|(0<<DP)|(0<<COL),
   //
  (0<<A_)|(0<<B_)|(0<<C_)|(1<<D_)|(1<<E_)|(0<<F_)|(1<<G_)|(0<<H_)|(0<<J_)|(0<<K_)|(1<<M_)|(0<<N_)|(0<<P_)|(0<<Q_)|(0<<DP)|(0<<COL)
};

//*******************************************************************************************************************;
void LCD_Init(void)
{
//   LCD, RTC:
  CLK_PCKENR2_bit.PCKEN23 = 1;
  while(CLK_ICKCR_bit.LSIRDY==0);
 
/*............. LCD.............*/
// :
 LCD_CR1_bit.DUTY1 = 1;/**** ****/
 LCD_CR1_bit.DUTY0 = 1;/* COM-*/
 
 LCD_CR2_bit.PON0 = 1;
 LCD_CR2_bit.PON1 = 1;
 LCD_CR2_bit.PON2 = 1;
 
// n = 2 (   2^n, F=(F_RTC/2)/2^n)
 //LCD_FRQ_bit.PS3 = 0;/****  (32768/2)/4=4096****/
 //LCD_FRQ_bit.PS2 = 0;/**      16**/
 LCD_FRQ_bit.PS1 = 1;/******     256 ******/
 //LCD_FRQ_bit.PS0 = 0;/******    256/4 *****/
// LCD:
 LCD_CR3_bit.LCDEN=1;
   
// 24     (SEG00 -- SEG23):
  LCD_PM0 = (1<<0)|(1<<1)|(1<<2)|(1<<3)|(1<<4)|(1<<5)|(1<<6)|(1<<7);
  LCD_PM1 = (1<<0)|(1<<1)|(1<<2)|(1<<3)|(1<<4)|(1<<5)|(1<<6)|(1<<7);
  LCD_PM2 = (1<<0)|(1<<1)|(1<<2)|(1<<3)|(1<<4)|(1<<5)|(1<<6)|(1<<7);
}
//*******************************************************************************************************************

void LCD_Contrast(unsigned char cntr)
{
// :
 LCD_CR2 |= ((cntr&0x07)<<1);
}
//*******************************************************************************************************************

static unsigned int LCD_Char_Find(unsigned char* symb)
{
  unsigned int cur_char;
// :
      if ( (*symb <= 0x39) && (*symb >= 0x30) )
      {
        cur_char = number_mask[*symb-'0'];
      }
//   :
      /*      */
      else{if ((*symb <= 0x5A) && (*symb >= 0x41))
      {
        cur_char = char_mask[*symb-'A'];
      }
      /*      */
      else{if ((*symb <= 0x7A) && ( *symb >= 0x61))    
      {
        cur_char = char_mask[*symb-'a'];
      }
//  :
      /*      */
      else{if ((*symb <= 0xDF) && ( *symb >= 0xC0))
      {
        cur_char = rus_char_mask[*symb-''];
      }
      /*      */
      else{if (*symb >= 0xE0)
      {
        cur_char = rus_char_mask[*symb-''];
      }
      else
      {
        switch (*symb)
        {
// :            
          case '$':
            cur_char = spec_char_mask[0];
            break;
            
          case '(':
            cur_char = spec_char_mask[1];
            break;
            
          case ')':
            cur_char = spec_char_mask[2];
            break;
            
          case '*':
            cur_char = spec_char_mask[3];
            break;
            
          case '+':
            cur_char = spec_char_mask[4];
            break;
            
          case '-':
            cur_char = spec_char_mask[5];
            break;
            
          case '=':
            cur_char = spec_char_mask[6];
            break;
            
          default:
            cur_char = 0x00;
            break;
        };
      };
      };
      };
      };
      };
      
      if(*symb=='m') {cur_char = spec_char_mask[7];}
      if(*symb=='u') {cur_char = spec_char_mask[8];}
      if(*symb=='n') {cur_char = spec_char_mask[9];}
      if(*symb=='') {cur_char = spec_char_mask[10];}
      
  return(cur_char);
}
//*******************************************************************************************************************

void LCD_Write_Char(unsigned char* ch, bool point, bool column, unsigned char position)
{
  unsigned int cur_char; /*    */
  unsigned char temp[2];
  
/*    */
  cur_char = LCD_Char_Find(ch);
  if(position<5)/*  ,   BAR */
  {
    /*     */
    if(point){cur_char |= (1<<DP);};
    /*    */
    if(column){cur_char |= (1<<COL);};
  };

temp[0]=(unsigned char)cur_char;
temp[1]=(unsigned char)(cur_char>>8);

/*     LCD_RAM,   */
switch (position)
  {
    /*  1 */
    case 1:
      /* 1M, 1E*/            /*   1M, 1E*/
      LCD_RAM0 &= ~((1<<1)|(1<<0)); LCD_RAM0 |= temp[0] & ((1<<1)|(1<<0));
      /* 1G, 1B*/            /*   1G, 1B*/
      LCD_RAM2 &= ~((1<<7)|(1<<6)); LCD_RAM2 |= (temp[0]<<4) & ((1<<7)|(1<<6));
      /* 1C, 1D*/            /*   1C, 1D*/
      LCD_RAM3 &= ~((1<<5)|(1<<4)); LCD_RAM3 |= temp[0] & ((1<<5)|(1<<4));
      /* 1F, 1A*/            /*   1F, 1A*/
      LCD_RAM6 &= ~((1<<3)|(1<<2)); LCD_RAM6 |= (temp[0]>>4) & ((1<<3)|(1<<2));
      /* 1COL, 1P*/          /*   1COL, 1P*/
      LCD_RAM7 &= ~((1<<1)|(1<<0)); LCD_RAM7 |= temp[1] & ((1<<1)|(1<<0));
      /* 1Q, 1K*/            /*   1Q, 1K*/
      LCD_RAM9 &= ~((1<<7)|(1<<6)); LCD_RAM9 |= (temp[1]<<4) & ((1<<7)|(1<<6));
      /* 1DP, 1N*/           /*   1DP, 1N*/
      LCD_RAM10 &= ~((1<<5)|(1<<4)); LCD_RAM10 |= temp[1] & ((1<<5)|(1<<4));
      /* 1H, 1J*/            /*   1H, 1J*/
      LCD_RAM13 &= ~((1<<3)|(1<<2)); LCD_RAM13 |= (temp[1]>>4) & ((1<<3)|(1<<2));
      break;
    
    /*  2 */
    case 2:
      /* 2M, 2E*/            /*   2M, 2E*/
      LCD_RAM0 &= ~((1<<3)|(1<<2)); LCD_RAM0 |= (temp[0]<<2) & ((1<<3)|(1<<2));
      /* 2G, 2B*/            /*   2G, 2B*/
      LCD_RAM2 &= ~((1<<5)|(1<<4)); LCD_RAM2 |= (temp[0]<<2) & ((1<<5)|(1<<4));
      /* 2C, 2D*/            /*   2C, 2D*/
      LCD_RAM3 &= ~((1<<7)|(1<<6)); LCD_RAM3 |= (temp[0]<<2) & ((1<<7)|(1<<6));
      /* 2F, 2A*/            /*   2F, 2A*/
      LCD_RAM6 &= ~((1<<1)|(1<<0)); LCD_RAM6 |= (temp[0]>>6) & ((1<<1)|(1<<0));
      /* 2COL, 2P*/          /*   2COL, 2P*/
      LCD_RAM7 &= ~((1<<3)|(1<<2)); LCD_RAM7 |= (temp[1]<<2) & ((1<<3)|(1<<2));
      /* 2Q, 2K*/            /*   2Q, 2K*/
      LCD_RAM9 &= ~((1<<5)|(1<<4)); LCD_RAM9 |= (temp[1]<<2) & ((1<<5)|(1<<4));
      /* 2DP, 2N*/           /*   2DP, 2N*/
      LCD_RAM10 &= ~((1<<7)|(1<<6)); LCD_RAM10 |= (temp[1]<<2) & ((1<<7)|(1<<6));
      /* 2H, 2J*/            /*   2H, 2J*/
      LCD_RAM13 &= ~((1<<1)|(1<<0)); LCD_RAM13 |= (temp[1]>>6) & ((1<<1)|(1<<0));
      break;
    
    /*  3 */
    case 3:
      /* 3M, 3E*/            /*   3M, 3E*/
      LCD_RAM0 &= ~((1<<5)|(1<<4)); LCD_RAM0 |= (temp[0]<<4) & ((1<<5)|(1<<4));
      /* 3G, 3B*/            /*   3G, 3B*/
      LCD_RAM2 &= ~((1<<3)|(1<<2)); LCD_RAM2 |= temp[0] & ((1<<3)|(1<<2));
      /* 3C, 3D*/            /*   3C, 3D*/
      LCD_RAM4 &= ~((1<<1)|(1<<0)); LCD_RAM4 |= (temp[0]>>4) & ((1<<1)|(1<<0));
      /* 3F, 3A*/            /*   3F, 3A*/
      LCD_RAM5 &= ~((1<<7)|(1<<6)); LCD_RAM5 |= temp[0] & ((1<<7)|(1<<6));
      /* 3COL, 3P*/          /*   3COL, 3P*/
      LCD_RAM7 &= ~((1<<5)|(1<<4)); LCD_RAM7 |= (temp[1]<<4) & ((1<<5)|(1<<4));
      /* 3Q, 3K*/            /*   3Q, 3K*/
      LCD_RAM9 &= ~((1<<3)|(1<<2)); LCD_RAM9 |= temp[1] & ((1<<3)|(1<<2));
      /* 3DP, 3N*/           /*   3DP, 3N*/
      LCD_RAM11 &= ~((1<<1)|(1<<0)); LCD_RAM11 |= (temp[1]>>4) & ((1<<1)|(1<<0));
      /* 3H, 3J*/            /*   3H, 3J*/
      LCD_RAM12 &= ~((1<<7)|(1<<6)); LCD_RAM12 |= temp[1] & ((1<<7)|(1<<6));
      break;
    
    /*  4 */
    case 4:
      /* 4M, 4E*/            /*   4M, 4E*/
      LCD_RAM0 &= ~((1<<7)|(1<<6)); LCD_RAM0 |= (temp[0]<<6) & ((1<<7)|(1<<6));
      /* 4G, 4B*/            /*   4G, 4B*/
      LCD_RAM2 &= ~((1<<1)|(1<<0)); LCD_RAM2 |= (temp[0]>>2) & ((1<<1)|(1<<0));
      /* 4C, 4D*/            /*   4C, 4D*/
      LCD_RAM4 &= ~((1<<3)|(1<<2)); LCD_RAM4 |= (temp[0]>>2) & ((1<<3)|(1<<2));
      /* 4F, 4A*/            /*   4F, 4A*/
      LCD_RAM5 &= ~((1<<5)|(1<<4)); LCD_RAM5 |= (temp[0]>>2) & ((1<<5)|(1<<4));
      /* 4COL, 4P*/          /*   4COL, 4P*/
      LCD_RAM7 &= ~((1<<7)|(1<<6)); LCD_RAM7 |= (temp[1]<<6) & ((1<<7)|(1<<6));
      /* 4Q, 4K*/            /*   4Q, 4K*/
      LCD_RAM9 &= ~((1<<1)|(1<<0)); LCD_RAM9 |= (temp[1]>>2) & ((1<<1)|(1<<0));
      /* 4DP, 4N*/           /*   4DP, 4N*/
      LCD_RAM11 &= ~((1<<3)|(1<<2)); LCD_RAM11 |= (temp[1]>>2) & ((1<<3)|(1<<2));
      /* 4H, 4J*/            /*   4H, 4J*/
      LCD_RAM12 &= ~((1<<5)|(1<<4)); LCD_RAM12 |= (temp[1]>>2) & ((1<<5)|(1<<4));
      break;
    
    /*  5 */
    case 5:
      /* 5M, 5E*/            /*   5M, 5E*/
      LCD_RAM1 &= ~((1<<1)|(1<<0)); LCD_RAM1 |= temp[0] & ((1<<1)|(1<<0));
      /* 5G, 5B*/            /*   5G, 5B*/
      LCD_RAM1 &= ~((1<<7)|(1<<6)); LCD_RAM1 |= (temp[0]<<4) & ((1<<7)|(1<<6));
      /* 5C, 5D*/            /*   5C, 5D*/
      LCD_RAM4 &= ~((1<<5)|(1<<4)); LCD_RAM4 |= temp[0] & ((1<<5)|(1<<4));
      /* 5F, 5A*/            /*   5F, 5A*/
      LCD_RAM5 &= ~((1<<3)|(1<<2)); LCD_RAM5 |= (temp[0]>>4) & ((1<<3)|(1<<2));
      /* 5COL, 5P*/          /*   5COL, 5P*/
      LCD_RAM8 &= ~((1<<1)|(1<<0)); LCD_RAM8 |= temp[1] & ((1<<1)|(1<<0));
      /* 5Q, 5K*/            /*   5Q, 5K*/
      LCD_RAM8 &= ~((1<<7)|(1<<6)); LCD_RAM8 |= (temp[1]<<4) & ((1<<7)|(1<<6));
      /* 5DP, 5N*/           /*   5DP, 5N*/
      LCD_RAM11 &= ~((1<<5)|(1<<4)); LCD_RAM11 |= temp[1] & ((1<<5)|(1<<4));
      /* 5H, 5J*/            /*   5H, 5J*/
      LCD_RAM12 &= ~((1<<3)|(1<<2)); LCD_RAM12 |= (temp[1]>>4) & ((1<<3)|(1<<2));
      break;
    
    /*  6 */
    case 6:
      /* 6M, 6E*/            /*   6M, 6E*/
      LCD_RAM1 &= ~((1<<3)|(1<<2)); LCD_RAM1 |= (temp[0]<<2) & ((1<<3)|(1<<2));
      /* 6G, 6B*/            /*   6G, 6B*/
      LCD_RAM1 &= ~((1<<5)|(1<<4)); LCD_RAM1 |= (temp[0]<<2) & ((1<<5)|(1<<4));
      /* 6C, 6D*/            /*   6C, 6D*/
      LCD_RAM4 &= ~((1<<7)|(1<<6)); LCD_RAM4 |= (temp[0]<<2) & ((1<<7)|(1<<6));
      /* 6F, 6A*/            /*   6F, 6A*/
      LCD_RAM5 &= ~((1<<1)|(1<<0)); LCD_RAM5 |= (temp[0]>>6) & ((1<<1)|(1<<0));
      /* 6COL, 6P*/          /*   6COL, 6P*/
      LCD_RAM8 &= ~((1<<3)|(1<<2)); LCD_RAM8 |= (temp[1]<<2) & ((1<<3)|(1<<2));
      /* 6Q, 6K*/            /*   6Q, 6K*/
      LCD_RAM8 &= ~((1<<5)|(1<<4)); LCD_RAM8 |= (temp[1]<<2) & ((1<<5)|(1<<4));
      /* 6DP, 6N*/           /*   6DP, 6N*/
      LCD_RAM11 &= ~((1<<7)|(1<<6)); LCD_RAM11 |= (temp[1]<<2) & ((1<<7)|(1<<6));
      /* 6H, 6J*/            /*   6H, 6J*/
      LCD_RAM12 &= ~((1<<1)|(1<<0)); LCD_RAM12 |= (temp[1]>>6) & ((1<<1)|(1<<0));
      break;
    /*      (  )*/
      default:
      break;
  }
}
//*******************************************************************************************************************

void LCD_Write_String(unsigned char* str)
{ unsigned char i = 1;

  /*      
        6 */
  while ((*str != 0) && (i < 7))
  {
    /*      */
    LCD_Write_Char(str, 0, 0, i);

    if(*str == '.')
    {
      i--;
      LCD_Write_Char(str-1, 1, 0, i);
    };
    if(*str == ':')
    {
      i--;
      LCD_Write_Char(str-1, 0, 1, i);
    };
    
    /*    .  */
    str++;

    /*    */
    i++;
  };
}


void LCD_Write_Int( long int* n, char point )
{
  long int temp = *n;
  unsigned char buf[6] = { 0, 0, 0, 0, 0, 0 };
  for ( int i = 0; i <= 5 ; i++ )
  {
    buf[5-i] = '0' + temp % 10;
    temp = ( temp - ( buf[5-i]-'0') ) / 10;
  }
  for ( int i = 0; i < 6; i++ )
  {
    if ( point == i ) 
      LCD_Write_Char( &buf[i], 1, 0, i+1 );
    else
      LCD_Write_Char( &buf[i], 0, 0, i+1 );
  }
}


void LCD_Write_Hex( long int* n )
{
  long int temp = *n;
  unsigned char digit;
  unsigned char buf[6] = { 0, 0, 0, 0, 0, 0 };
  for ( int i = 0; i <= 5 ; i++ )
  {
    digit = temp % 16;
    if ( digit < 10 )
      buf[5-i] = '0' + temp % 16;
    else
      buf[5-i] = 'A' + temp % 16 - 10;
      
    temp = ( temp - digit ) / 16;
  }
  for ( int i = 0; i < 6; i++ )
  {
    LCD_Write_Char( &buf[i], 0, 0, i+1 );
  }
}

void LCD_eWrite_Int( long int* n, char point, int l, int x )
{
  char c = 0;
  long int temp = *n;
  unsigned char buf[6] = { 0, 0, 0, 0, 0, 0 };
  for ( int i = 5-x; i <= 5 ; i++ )
  {
    if ( ( temp % 10 == 0 ) && ( c>=l ) )
        break;      
    buf[5-i] = '0' + temp % 10;
    temp = ( temp - ( buf[5-i]-'0') ) / 10;
    c++;
  }

  if ( l == -1 )
  {
    for ( int i = 0; buf[i] == '0'; i++ )
      buf[i] = 0;
  }
  for ( int i = 0; i < 6; i++ )
  {
    if ( point == i ) 
      LCD_Write_Char( &buf[i], 1, 0, i+1 );
    else
      LCD_Write_Char( &buf[i], 0, 0, i+1 );
  }
}

void LCD_Scroll(unsigned char* str)
{  
  do
  {
    LCD_Write_String(str);
    str ++;
    for ( long int i = 0; i < 10000; i++ );
  }
  while ( *(str+5) != 0 );
}

void show_All( void )
{
  LCD_RAM0 = 0xFF;
  LCD_RAM1 = 0xFF;
  LCD_RAM2 = 0xFF;
  LCD_RAM3 = 0xFF;
  LCD_RAM4 = 0xFF;
  LCD_RAM5 = 0xFF;
  LCD_RAM6 = 0xFF;
  LCD_RAM7 = 0xFF;
  LCD_RAM8 = 0xFF;
  LCD_RAM9 = 0xFF;
  LCD_RAM10 = 0xFF;
  LCD_RAM11 = 0xFF;
  LCD_RAM12 = 0xFF;
  LCD_RAM13 = 0xFF;
}